local self = AutoRoller

--  db[info[#info]]

function self:OptionGet(info)
   return self.db.profile[info[#info]]
end

function self:OptionSet(info, value)
   self.db.profile[info[#info]] = value
end

function self:RuleGet(info)
   return self.db.profile.rules[self.ruleNameLUT[info[#info-1]]][info[#info]]
end

function self:RuleSet(info, value)
   self.db.profile.rules[self.ruleNameLUT[info[#info-1]]][info[#info]] = value
end

function self:DeleteRule(info)
   local rules = self.db.profile.rules
   local lut = self.ruleNameLUT

   self.optionTable.args.rules.args[lut[#rules]] = nil --remove last rule

   tremove(rules, lut[info[#info-1]])
end

function self:MoveRuleUp(info)
   --only have to swap rules in db, optionsTable will automatically adjust
   local n = self.ruleNameLUT[info[#info-1]]

   if n == 1 then return end

   local rules = self.db.profile.rules

   rules[n],rules[n-1] = rules[n-1],rules[n]
end

function self:MoveRuleDown(info)
   --only have to swap rules in db, optionsTable will automatically adjust
   local n = self.ruleNameLUT[info[#info-1]]
   local rules = self.db.profile.rules

   if n == #rules then return end

   rules[n],rules[n+1] = rules[n+1],rules[n]
end

function self:NewRule()
   local rules, lut = self.db.profile.rules, self.ruleNameLUT
   local n = #rules+1
   local name = 'rule'..n
   lut[n], lut[name] = name, n
   rules[n] = {
      enabled = false,
      action = 3,
      logic = "",
   }
   self:BuildRuleOption(n, name)
end

self.optionTable = {
   name = "Auto Roller",
   desc = "Automatically roll on items.",
   type = 'group',
   handler = self,
   get = 'OptionGet',
   set = 'OptionSet',
   args = {
      desc = {
         name = "Automatically roll on items.",
         type = 'description',
         order = 100,
         width = 'full',
      },
      enabled = {
         name = "Enabled",
         type = 'toggle',
         order = 200,
         get = 'IsEnabled',
         width = 'full',
         set = function (info, v)
            local self = info.handler
            self.db.profile.enabled = v
            if v then
               self:Enable()
            else
               self:Disable()
            end
         end,
      },
      chat = {
         name = "Chat Output",
         desc = "Output to chat when automatically rolling on an item.",
         type = 'toggle',
         order = 300,
         width = 'full',
      },
      newRule = {
         name = "New Rule",
         type = 'execute',
         order = 400,
         func = 'NewRule',
      },
      rules = {
         name = "Rules",
         type = 'group',
         inline = true,
         order = 500,
         set = 'RuleSet',
         get = 'RuleGet',
         func = 'RuleFunc',
         args = {
            --rules go here
         },
      },
      help = {
         name = [[
Each rule's logic can be any statement that evaulates to true or false.
There are several |cffEF9C00tags|r available to ease use.

The following values are constant, and used for the quality tag:

|cffEF9C00poor|r = |cff00FFFF0|r
|cffEF9C00common|r = |cff00FFFF1|r
|cffEF9C00uncommon|r = |cff00FFFF2|r
|cffEF9C00rare|r = |cff00FFFF3|r
|cffEF9C00epic|r = |cff00FFFF4|r
|cffEF9C00legendary|r = |cff00FFFF5|r
|cffEF9C00artifact|r = |cff00FFFF6|r
|cffEF9C00heirloom|r = |cff00FFFF7|r

The fallowing will change depending on the item being rolled on:

|cffEF9C00raid|r = |cffFF7FFFtrue|r if you are in a raid
|cffEF9C00party|r = |cffFF7FFFfalse|r if you are in a raid
|cffEF9C00combat|r = |cffFF7FFFtrue|r if you are in combat
|cffEF9C00quality|r = |cff00FFFF0|r to |cff00FFFF7|r or use the tags above
|cffEF9C00itype|r = |cff00FFFFArmor|r, |cff00FFFFConsumable, |cff00FFFFWeapon, ...  Case sensitive
|cffEF9C00subtype|r = see http://www.wowwiki.com/ItemType
|cffEF9C00ilvl|r = item level, |cff00FFFF226|r from ulduar
|cffEF9C00reqlvl|r = required level to wear item
|cffEF9C00name|r = name of item, case sensitive
|cffEF9C00id|r = itemID
|cffEF9C00boe|r = |cffFF7FFFtrue|r if item is bind on equip
|cffEF9C00bop|r = |cffFF7FFFtrue|r if item is bind on pickup


All of the preceding tags must be lower case, and can be combined with '|cffFF7FFFand|r', '|cffFF7FFFor|r', and '|cffFF7FFFnot|r' along with parenthesis.  Following are some examples:

Roll if the item is bind on equip and a green (uncommon) or blue (rare):

|cffEF9C00boe|r |cffFF7FFFand|r (|cffEF9C00quality|r == |cffEF9C00uncommon|r |cffFF7FFFor|r |cffEF9C00quality|r == |cffEF9C00rare|r)


If in a raid and in combat, for epic BoP items:

|cffEF9C00raid|r |cffFF7FFFand|r |cffEF9C00combat|r |cffFF7FFFand|r |cffEF9C00bop|r |cffFF7FFFand|r (|cffEF9C00quality|r == |cffEF9C00epic|r)


If your character's name is Jimbo, ilvl is 226 or above, and it is not armor:

UnitName(|cff00FFFF'player'|r) == |cff00FFFF"Jimbo"|r |cffFF7FFFand|r |cffEF9C00ilvl|r >= |cff00FFFF226|r |cffFF7FFFand|r |cffFF7FFFnot|r (|cffEF9C00itype|r == |cff00FFFF"Armor"|r)


]],
         type = 'description',
         order = 600,
         width = 'full',
      }
   }
}

--
local ruleArgs = {
   enabled = {
      name = "Enabled",
      type = 'toggle',
      order = 100,
      width = 'half',
   },
   action = {
      name = "Action",
      type = 'select',
      values = {[0] = "pass", [1] = "need", [2] = "greed", [3] = "nothing"},
      order = 101,
      width = 'half',
   },
   delete = {
      name = "Delete",
      type = 'execute',
      order = 103,
      width = 'half',
      confirm = true,
      func = 'DeleteRule',
   },
   move = {
      name = "Move:",
      type = 'description',
      order = 200,
      width = 'half',
   },
   moveup = {
      name = "Up",
      type = 'execute',
      order = 201,
      width = 'half',
      func = 'MoveRuleUp',
   },
   movedown = {
      name = "Down",
      type = 'execute',
      order = 202,
      width = 'half',
      func = 'MoveRuleDown',
   },
   logic = {
      name = "Logic",
      type = 'input',
      order = 300,
      width = 'full',
      validate = 'ValidateLogic',
   },
}

function self:BuildRuleOption(n, name)
   local args = self.optionTable.args.rules.args

   args[name] = {
      type = 'group',
      name = "Rule"..n,
      order = 100+n,
      args = ruleArgs,
   }
end

function self:RegisterOptions()
   local options = self.optionTable
   local profile = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)

   local registry = LibStub('AceConfigRegistry-3.0')
   local dialog = LibStub('AceConfigDialog-3.0')
   LibStub('AceConfig-3.0'):RegisterOptionsTable('AutoRoller', options, 'autoroller')

	registry:RegisterOptionsTable('AutoRoller Options', options)
	registry:RegisterOptionsTable('AutoRoller Profiles', profile)

   local main = dialog:AddToBlizOptions('AutoRoller Options', 'AutoRoller')
   self.optionsFrames = {
      main = main,
		profiles = dialog:AddToBlizOptions('AutoRoller Profiles', 'Profiles', 'AutoRoller'),
   }

   if self.dataobj then
      self.dataobj.OnClick = function() InterfaceOptionsFrame_OpenToCategory(main) end
   end
end
